#!/bin/ksh
#> This script will install Prospect 8.0 / Nortel MTX15 RP16 HF02 hotfix
#> Usage: install.sh {-help|-install|-uninstall}
#

InstallFileset='
vendor/Nortel/MSC/1.3/1xEVDO/scripts/preParse1x.pl
'

########################################################
# set paths and files

RELEASE="15.0.16.0.2"
SourceDir="${FLEXPM_HOME}/patches/${RELEASE}"
SourceDir1="${FLEXPM_HOME}/patches/${RELEASE}/nortel"
BackupDir1="${FLEXPM_HOME}/patches/${RELEASE}/backup/nortel"

########################################################

### give help
  ToolName=`basename $0`
  runTime=`date '+%Y%m%d%H%M'`
  case $1 in
  -help|-usage)
    echo "${ToolName} usage information -- " 1>&2
    grep "^#> " $0 1>&2
    grep "^#>  \"" $0 1>&2
    exit 2
    ;;
  esac

# Check env settings

  if [ $# -ne 1 ]; then
    echo "Usage $0 {-help | -install | -uninstall}"
    exit;
  fi

  echo "Checking environment....."
  if ! [[ -d ${FLEXPM_HOME} ]]; then
    echo "Env variable FLEXPM_HOME must be set to appropriate dir before installation."
    exit;
  fi

  
cd $SourceDir

# Source and destination directories

  case $1 in
  -install)
    echo "Installing patch..."

    echo "Files to be updated/added ..."
    for f in ${InstallFileset}
    do
        echo "${f}"
        if [[ -a ${FLEXPM_HOME}/${f} ]]; then
        # permissions
        chmod u+w ${FLEXPM_HOME}/${f}
		
        # backup
        cp -pf ${FLEXPM_HOME}/${f} ${BackupDir1}/${f}
        tar -cf backup.$runTime.tar ${BackupDir1}
		
        fi
        # install files
	cp -pf ${SourceDir1}/${f} ${FLEXPM_HOME}/${f}
		
	# permissions
	chmod u-w ${FLEXPM_HOME}/${f}
	chmod u+x ${FLEXPM_HOME}/${f}
    done

    echo "Installation completed successfully."
    ;;

  -uninstall)

    echo " Contact Customer support for uninstallation of the patch "

    ;; 

  * )
    echo "Unknown option"
    echo "Usage: install.sh {-help|-install|-uninstall}"
    ;;
  esac
