#!/usr/bin/env perl
# @(#) preParse1x.pl 1.4.0 03/11/17 13:57:46
# @(#) 
# PreParse1x.pl
##########################################
# This script preparses 1xEVDO datafiles
# It performs three main tasks:
# 1. Formats the data into loader supported
#    pipe-delimited format
# 2. Builds full keys for each instance
#    via mapping from database and static lookups
# 3. Normalizes timestamps to the nearest
#    15 minute period
#
# Usage: preParse1x.pl <datafile> <systemid> <mscid>
##########################################
#use lib "$ENV{ FLEXPM_HOME }/vendor/Nortel/MSC/1.3/1xEVDO/scripts/";
#use Tool;
use Data::Dumper;
#use locale;

sub fillInBitMapArr($);
###########################################
# Support the customer configured templates
# 2004-08-04

$minConfigField=2;
$templateFile="\.\.\/\.\.\/\.\.\/1.3\/1xEVDO\/scripts\/templateFile";
$configFile="\.\.\/\.\.\/\.\.\/1.3\/1xEVDO\/scripts\/configFile";
my %templateEntityHash =();
my $nodename_temp;
my $g_templ_name="";
my %minMaxAvgHash;

if ( -s $templateFile ) {
   open (templateConfFile, $templateFile);

   while ($line =<templateConfFile>)  {
     chomp($line);

     # skip comments
     if ($line !~ /^#/) {
       @lineWords = ();
       @lineWords = split (/\|/, $line);

       if (@lineWords >= $minConfigField) {
         $configTemplate = shift @lineWords;
         $configEntity = shift @lineWords;
         $templateEntityHash{$configTemplate} =  $configEntity;
         $reference_templatehash{$configTemplate} = shift @lineWords; 
		 $ref_instancehash{$configTemplate} = shift @lineWords;
       }
       else {
         print "Missing fields at line $line in templateFile\n";
       }
     }
   }
   close (templateConfFile);

} else {
  print " No user configured templates available, use default templates to do the processing\n\n";

}

# Reading the config file newly added to get the remediation type

if ( -s $configFile ) {
   open (pegConfFile, $configFile);

   while ($line =<pegConfFile>)  {
     chomp($line);

     # skip comments
     if ($line !~ /^#/) {
       @lineWords = ();
       @lineWords = split (/\|/, $line);

	   if ( uc($lineWords[2]) ne 'R' && uc($lineWords[2]) ne 'N' ) {
		   $entityandpeg = uc($lineWords[0])."-".$lineWords[1];
		   $configHash{uc($entityandpeg)} = $line;
		   
	   }
       if (@lineWords >= 0) {
         $configEntity = shift @lineWords;
         $pegName      = shift @lineWords;
	 $rem_type     = shift @lineWords;
         $reference_peghash{lc($configEntity).'-'.lc($pegName)} =  $rem_type;
       }
       else {
         print "Missing fields at line $line in configFile\n";
       }
     }
   }
   close (pegConfFile);
   %configHash1=%configHash;

} else {
  print " No user configured pegs available\n\n";

}

###########################################
# Build static lookup hashes
%RNCSlotHash = ('1' => 'BIOCard_Slot1|10101|65793',
'2' => 'BIOCard_Slot2|10201|66049',
'3' => 'RNSM_Slot3|10301|66305',
'4' => 'RNSM_Slot4|10401|66561',
'5' => 'RNSM_Slot5|10501|66817',
'6' => 'RNSM_Slot6|10601|67073',
'7' => 'SC_Slot7|10701|67329',
'8' => 'Unused|N.A|N.A',
'9' => 'SC_Slot9|10901|67841',
'10' => 'Unused|N.A|N.A',
'11' => 'BIOCard_Slot11|10b01|68353',
'12' => 'BIOCard_Slot12|10c01|68609',
'13' => 'RNSM_Slot13|10d01|68865',
'14' => 'RNSM_Slot14|10e01|69121',
'15' => 'RNSM_Slot15|10f01|69377',
'16' => 'RNSM_Slot16|11001|69633'
);

%RNCCPUHash = ('65793' => 'BIOCard_Slot1|10101',
'66049' => 'BIOCard_Slot2|10201',
'66305' => 'RNSM_Slot3|10301',
'66561' => 'RNSM_Slot4|10401',
'66817' => 'RNSM_Slot5|10501',
'67073' => 'RNSM_Slot6|10601',
'67329' => 'SC_Slot7|10701',
'67841' => 'SC_Slot9|10901',
'68353' => 'BIOCard_Slot11|10b01',
'68609' => 'BIOCard_Slot12|10c01',
'68865' => 'RNSM_Slot13|10d01',
'69121' => 'RNSM_Slot14|10e01',
'69377' => 'RNSM_Slot15|10f01',
'69633' => 'RNSM_Slot16|11001'
);

%DOMSlotHash = (
'3' => 'BIOSC_Slot3|10301|66305',
'4' => 'DOM_Modem_Slot4|10401 (Forward Link Modem-FLM)|66561|Forward Link Modem',
'4' => 'DOM_Modem_Slot4|10402 (Reverse Link Modem-RLM)|66562|Reverse Link Modem'
);

%DOMCPUHash =(
'66305' => 'BIOSC_Slot3|10301',
'66561' => 'DOM_Modem_Slot4|10401',
'66562' => 'DOM_Modem_Slot4|10402'
);

%DOMSectHash = (
'element1' => 'X',
'element2' => 'Y',
'element3' => 'Z'
);


  exists $ENV{ DB_CONNECT } or die "Error: DB_CONNECT environment variable not set\n";
  $connectString = $ENV{ DB_CONNECT };
  exists $ENV{ PROSPECT_HOME } or die "Error: PROSPECT_HOME environment variable not set\n";
  $prospectHome = $ENV{ PROSPECT_HOME };

#Get the system id and msc id (DO_RNC Id) from loader.sh 
my $inputFile = $ARGV[0];
my @fileNameArray = split(/\./, $inputFile);
my $fnParts = @fileNameArray;
#Changes for SEAde64443 (changed 5 to 4 below and die command below) 
if ($fnParts != 4) {
#  die "file name: $inputFile should be <RNC_ID>.<TIMESTAMP>.<COLLECTION_PERIOD>.RNCpm.extract \n";
  die "file name: $inputFile should be <RNC_ID>.<TIMESTAMP>.<COLLECTION_PERIOD>.RNCpm\n";
}
# Timestamp from filename is used in queries on mapping tables
# but not in the output data
#DO_RNC_Id and systemId should be passed by loader.sh
$fnTstamp = $fileNameArray[1];
$systemId = $ARGV[1];
$DO_RNC_Id = $ARGV[2];
$debug = $ARGV[3];

#open the input and output files
unless (open(INFILE, "$inputFile")) {
  die("unable to open $inputFile for reading \n");
}

unless (open(OUTFILE, ">$fileNameArray[0].$fileNameArray[1].$fileNameArray[2].$fileNameArray[3].pp")) {
  die("unable to open $fileNameArray[0].$fileNameArray[1].$fileNameArray[2].$fileNameArray[3].pp for writing \n");
}

if ($debug == "1")
{
unless (open(DEBUGFILE, ">$fileNameArray[0].$fileNameArray[1].$fileNameArray[2].$fileNameArray[3].pp.debug")) {
  die("unable to open $fileNameArray[0].$fileNameArray[1].$fileNameArray[2].$fileNameArray[3].pp.debug for writing \n");
}
}
# work out the filename for the previous file
# and put it on the first line
# this is needed by the remediation script

###########################################
# get the all the mapping info from DB except
# for the If hash which needs the DO_RNC_Label
# and is performed below
($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
print "preParser:Reading from RNCCFG tables start: $theTime \n"; 
%DOMhash = buildLookupHash("DOM");
%IS856hash = buildLookupHash("IS856");
%PDSNhash = buildLookupHash("DO_PDSN");
#print Dumper  [%PDSNhash];
#Changes for SEAde64442
my %newH={};
my %newH1={};
my @retArr= buildLookupHash("IF");
my $hashRef1=$retArr[0];
my $hashRef2=$retArr[1];
%newH=%$hashRef1;
%newH1=%$hashRef2;

# end changes
%DOMSECThash = buildLookupHash("DOMSECT");
($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second";
print "preParser:Reading from RNCCFG tables end and start of main loop: $theTime \n"; 
$norm =1;

#This is added to avoid printing in duplicates ".pp" file
#%avoid_duplicate_print;
#Line by line:
while ($line = <INFILE>) 
  {
    next if (($line eq '') || ($line =~ /^\#/) || ($line =~ /^\t+$/));
###########################################
# Get the DO_RNC Id and DO_RNC Label
    if ($line =~ /^NodeName/) {
       $nodeName = collectNodeName($line);
       $nodename_temp="$nodeName";
       $norm =1;
    } else {
      if ($norm == 0) {
         goto PPP;
      }
    }

    if ($line =~ /^NodeAlias/) {
      chomp $line;
      @NodeAliasLine = split(/=/,$line);
      $nodeAliasName = $NodeAliasLine[1];
    }

###########################################
# Entity determined by template name
# assuming that ConfName reflects template name
# Only known templates are supported!

    if ($line =~ /^ConfName/ ) {
      chomp $line;
      my @entityLine = split(/=/,$line);
#      print "Templatename is $entityLine[1] \n";
      $g_templ_name=$entityLine[1];

# initialize dom_template.(Alltel fix for SEAde69249)
      $dom_template="";
#
	  ########## support DOM templates with entPhysicalIndex field
      if ( $ref_instancehash{$entityLine[1]} ne 'A' ) {
	  $dom_template = "domWithIns";
#	  print "setting dom_template = $dom_template and $ref_instancehash{$entityLine[1]} \n";
      }
      ########## support DOM templates with entPhysicalIndex field
      if ($templateEntityHash{"$entityLine[1]"} ne "") 
	{
	  $dataEntity = $templateEntityHash{"$entityLine[1]"};  
	  $ucdataEntity = uc $dataEntity; # uppercase ent name for parser...
	  $g_template_name = $entityLine[1]; #Muthu
	}
      else 
	{
 	 print OUTFILE ("# Data template $line is not a recognized template \n");
	 $norm =0;
	 goto PPP;
	}
    }
############################################
# Collect the variable names in the order they appear
    if ($line =~ /^OIDs/)
      { 
	splice @varNames, 0; #clear the varNames array
      }
    if ($line =~ /^Var-/) 
	{  
	  pushVarNameArray (\$line);
    }
############################################
# Data comes after ---------------------
    if ($line =~ /^---------/) 
      {
      ########## support DOM templates with entPhysicalIndex field
	if ($dom_template eq "domWithIns") {
		$numKeyFields = 2; # dom templates with entPhysicalIndex field
		$dom_template = ""; 
	} else {
		$numKeyFields = 1;
	}
	$numPegs = @varNames;
        if (!exists $minMaxAvgHash{$g_template_name})
        {
           fillInBitMapArr($ucdataEntity);
        }
	$numPegs = ($numPegs + $numKeyFields);
	if ($dataEntity =~ /DOM_If/) {
	  #Ifhash needs DO_RNC_Label All others need
	  #only system id and are completed previously
	   %IFhash={};
	   if (exists $newH{$nodeAliasName} )
	   {
	     $numElem=$newH{$nodeAliasName}->{cnt};
	     for ($i=0;$i<=$numElem;$i++)
	     {
		my $sArrPtr=$newH{$nodeAliasName}->{arr}->[$i]->{sarr};
		@sArr=@$sArrPtr;
		$keyname=$sArr[0];
		$valname=$sArr[1];
	        $IFhash{$keyname}=$valname;
	     }
	   }
	}
	elsif ($dataEntity =~ /DO_RNC_If/) {
	   %IFhash={};

	   if (exists $newH1{$nodeName} )
	   {
	     $numElem=$newH1{$nodeName}->{cnt};
	     for ($i=0;$i<=$numElem;$i++)
	     {
		my $sArrPtr=$newH1{$nodeName}->{arr}->[$i]->{sarr};
		@sArr=@$sArrPtr;
		$keyname=$sArr[0];
		$valname=$sArr[1];
	        $IFhash{$keyname}=$valname;
	     }
	   }
	  
	}
($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
#print "preParser:Start  of main loop that does Pivot: $theTime \n"; 
	while ($line=<INFILE>) {
		if ($line =~ /^NodeName/) { # if the next line is another template
		  $nodeName = collectNodeName($line);   # ensure NodeName gets set
		}
		last unless ($line =~ /^\d/); # unless the line starts with a digit character
		chomp $line;
		@dataLine = split(/\t+/, $line); #expecting tab delimited
		# The last count seems to have a blank at the end. eg. rncType and that does not allow jloader to load
		$dataLine[$#dataLine - 1 ]=~s/\s*$//g;
		### end fix
		$instance = $dataLine[0]; 
		$origTstamp = $dataLine[($numKeyFields - 1)];
		checkNumPegs ();
		$peg = $numKeyFields;
		#Build the key for the row
		$returnedKey =  buildKey($dataEntity, $instance);
		#Normalize the timestamp
		  my @timeArray = split(/[\/| |:|-]/,$origTstamp);
		  $interval = $reference_templatehash{$g_template_name};
		  $nearest_interval = $interval * int($timeArray[4] / $interval ); # this may return '0'
		  $nearest_interval = '00' if $nearest_interval eq '0';
		  $outTstamp = "$timeArray[2]$timeArray[0]$timeArray[1]$timeArray[3]$nearest_interval"."00";

###########################################
	    # Print out a row for each peg
		($ref_tstamp) = substr($outTstamp,8,4);

#             print "dataLineElemetns = $dataLineElements and so is $#dataLine \n";
	        if (($returnedKey =~ /\|\|/) || (length($returnedKey) == 0) )
                 {
      	              while ($peg < $dataLineElements) {	#iterate over fields within a row
#			push @main_array, ("# Key lookup failed for $ucdataEntity|$returnedKey|$outTstamp|$varNames[($peg - 2)]|$dataLine[$peg] \n");
			print  OUTFILE "# Key lookup failed for $ucdataEntity|$returnedKey|$outTstamp|$varNames[($peg - 2)]|$dataLine[$peg] \n";
                      $peg++;
                    }
		    next;
                 }
		 else
		 {
		     @minMaxAvg=();
		     $pminMaxAvg='';
		    if (exists $minMaxAvgHash{$g_template_name} )
		    {
                     $pminMaxAvg= $minMaxAvgHash{$g_template_name}{'Arr'} ;
		     @minMaxAvg=@{$pminMaxAvg};
		     }
	             while ($peg < $dataLineElements) {	#iterate over fields within a row
                    if ($numKeyFields == 1)
                    {
                       $pegIndex=($peg)-1;
                    }
                    elsif  ($numKeyFields == 2)
                    {
                       $pegIndex=($peg)-2;
                    }
#		    print "peg=$peg and numKeyFields = $numKeyFields \n";
#		    print "pegIndex = $pegIndex and $varNames[$pegIndex] and $dataLine[$peg]  \n";

                       if (uc($dataLine[$peg]) =~ "NOT AVAILABLE") { #peg values 'not available' in data
#		         push @main_array, ("#$ucdataEntity|$returnedKey|$outTstamp|$varNames[($pegIndex)]|$dataLine[$peg] \n");
			 $temp_pegname=lc($varNames[($pegIndex)]);
	                 $hkey =  '#'.$ref_tstamp.$result_hash{$g_template_name.':'.$temp_pegname}.'&&'.$ucdataEntity.'|'.$returnedKey.'|'.$outTstamp.'|'.$varNames[$pegIndex];
			 if (!exists $rem_dupl_hash{$hkey} )
			 {
                            $rem_dupl_hash{$hkey}  = 'NOT AVAILABLE';
			    undef $hkey;
		         }
			  $peg++;
			 next;
		        }
		       if ( exists $reference_peghash{lc($ucdataEntity).'-'.lc($varNames[($pegIndex)])} ) 
		       {# All OK

#                          @minMaxAvg= $minMaxAvgHash{$g_template_name} ;
			  $temp_pegname=lc($varNames[($pegIndex)]);
#			  print "pegIndex = $pegIndex and peg= $peg and pegname= $temp_pegname and flag=$minMaxAvg[$pegIndex] \n";
			 if ( $minMaxAvg[$pegIndex]  == 0) 
			 {
#			      print "into if part of loop for $temp_pegname \n";
#			      push @main_array,("$ref_tstamp$result_hash{$g_template_name.':'.$temp_pegname}&&$ucdataEntity|$returnedKey|$outTstamp|$varNames[($pegIndex)]|$dataLine[$peg] \n");

                             #Special DOM_Sector key processing for SEAde63717
			     #The PegNamewill get the  last 2 parts of the key because we need to load the 
			     # pegs with diff. keys into diff. columns in table.
			     if ( $ucdataEntity eq "DOM_SECTOR" )
			     {
#			      print "DOMSEC returned key = $returnedKey \n";
                                 @lastparts=split('\|',$returnedKey);
                                 @lastparts1=split('/',$lastparts[2]);
                                 @lastparts=split("_",$lastparts1[2]);
                                 if (@lastparts >0)
				 {
                                 shift  @lastparts; #get rid of 8_
                                 shift  @lastparts; #get rid of Sector Number as well
				 }
                                 if (@lastparts >0)
                                 {
#                                    $newVarStr=$varNames[($pegIndex)].'_'.join("_",@lastparts);
                                    if ($g_template_name =~ 'RnSector[_]?FtcStats' )
                                    {
				       $rate=shift @lastparts;
				       $rate=sprintf('%02d',$rate);
                                       $newVarStr=$varNames[($pegIndex)].'_PacketRate'. $rate;
				       $rate='';

				    }
				    elsif  ($g_template_name =~ 'RnSector[_]?RtcStats' )
				    {
				       $rate=shift @lastparts;
				       $rate=sprintf('%02d',$rate);
				       $subpacket=shift @lastparts;
				       $subpacket=sprintf('%d',$subpacket);
                                       $newVarStr=$varNames[($pegIndex)].'_Rate'. $rate .'SubPacket'. $subpacket;
				       $rate='';
				       $subpacket='';
				    }
				    elsif  ($g_template_name =~ 'RnSector[_]?AchStats' )
				    {
				       $size=shift @lastparts;
				       $size=sprintf('%d',$size);
				       $rate=shift @lastparts;
				       $rate=sprintf('%d',$rate);
                                       $newVarStr=$varNames[($pegIndex)].'_Size'. $size .'Rate'. $rate;
				       $size='';
				       $rate='';
				    }
				    elsif  ($g_template_name =~ 'RnSector[_]?CchStats') #obsolete as of R2.2
				    {
				       $size=shift @lastparts;
				       $size=sprintf('%d',$size);
                                       $newVarStr=$varNames[($pegIndex)].'_CapsuleSize'. $size;
				       $size='';
				    }
				    else
				    {
                                       $newVarStr=$varNames[($pegIndex)].'_'.join("_",$lastparts);
				    }
#			            print "$peg , $pegIndex varname $varNames[($pegIndex)] \n";
				    $hkey =  $ref_tstamp.$result_hash{$g_template_name.':'.$temp_pegname}.'&&'.$ucdataEntity.'|'.$returnedKey.'|'.$outTstamp.'|'.$newVarStr;
				    if ( "$debug" == 1 )
				    {
				       $debug_hkey=$g_template_name.'_'.$hkey;
				       $debug_hash{$debug_hkey} = $dataLine[$peg];
				       undef $debug_hkey;
				    }
				    if (!exists $rem_dupl_hash{$hkey} )
				    {
                                       $rem_dupl_hash{$hkey}  = $dataLine[$peg];
					undef $hkey;
				    }
                                 }
				 else
				 {
#				  print "Going into else lastparts for $temp_pegname  \n";
#			             print OUTFILE "$ref_tstamp$result_hash{$g_template_name.':'.$temp_pegname}&&$ucdataEntity|$returnedKey|$outTstamp|$varNames[($pegIndex)]|$dataLine[$peg] \n";
				    $hkey =  $ref_tstamp.$result_hash{$g_template_name.':'.$temp_pegname}.'&&'.$ucdataEntity.'|'.$returnedKey.'|'.$outTstamp.'|'.$varNames[$pegIndex];
				    
				    if ( "$debug" == 1 )
				    {
				       $debug_hkey=$g_template_name.'_'.$hkey;
				       $debug_hash{$debug_hkey} = $dataLine[$peg];
				       undef $debug_hkey;
				    }
				    if (!exists $rem_dupl_hash{$hkey} )
				    {
                                        $rem_dupl_hash{$hkey}  = $dataLine[$peg];
					undef $hkey;
				    }
#                                  $rem_dupl_hash{$ref_tstamp$result_hash{$g_template_name.':'.$temp_pegname}&&$ucdataEntity|$returnedKey|$outTstamp|$varNames[$pegIndex]|$dataLine[$peg]}  = $dataLine[$peg];
				 }
			      }
			      elsif ($ucdataEntity eq 'DO_RNC'  || $ucdataEntity eq 'DO_RNC_CARD')
			      {
				 @lastparts=();
                                 @lastparts=split(':',$returnedKey);
				 $atDescStr = ''; # initialize variable

				 if (@lastparts >1 )
				 {
				    $atDescOrRncPkt= pop @lastparts;
				    if ( $atDescOrRncPkt == 1 ) 
				    {
				      $atDescStr = '_Rev0'; 
				    }
				    elsif (  $atDescOrRncPkt == 2 )
				    {
				      $atDescStr = '_RevA';
				    }
				    elsif (  $atDescOrRncPkt eq "R1" )
				    {
				      $atDescStr = '_MFPA';
#				      $lastparts[1]=~s/R//;
				    }
				    elsif (  $atDescOrRncPkt eq "R2" )
				    {
				      $atDescStr = '_EMFPA';
#				      $lastparts[1]=~s/R//;
				    }
				    if ( uc($varNames[$pegIndex]) eq uc('atDescriptor') ||
				       uc($varNames[$pegIndex]) eq uc('rncSlotPacketApplication'))
				    {
				      $atDescStr = '';
				    }
				    $newDORNCKey=join("\|",@lastparts);
				    $hkey =  $ref_tstamp.$result_hash{$g_template_name.':'.$temp_pegname}.'&&'.$ucdataEntity.'|'.$newDORNCKey.'|'.$outTstamp.'|'.$varNames[$pegIndex].$atDescStr;
				 }
				 else
				 {
				      $hkey =  $ref_tstamp.$result_hash{$g_template_name.':'.$temp_pegname}.'&&'.$ucdataEntity.'|'.$returnedKey.'|'.$outTstamp.'|'.$varNames[$pegIndex];
				 }
				 
				 if ( "$debug" == 1 )
				 {
				       $debug_hkey=$g_template_name.'_'.$hkey;
				       $debug_hash{$debug_hkey} = $dataLine[$peg];
				       undef $debug_hkey;
				 }
				 if (!exists $rem_dupl_hash{$hkey} )
				 {
                                     $rem_dupl_hash{$hkey}  = $dataLine[$peg];
				     undef $hkey;
				 }
			      }
			      else
			      {
				    $hkey =  $ref_tstamp.$result_hash{$g_template_name.':'.$temp_pegname}.'&&'.$ucdataEntity.'|'.$returnedKey.'|'.$outTstamp.'|'.$varNames[$pegIndex];
				 
				    if ( "$debug" == 1 )
				    {
				       $debug_hkey=$g_template_name.'_'.$hkey;
				       $debug_hash{$debug_hkey} = $dataLine[$peg];
				       undef $debug_hkey;
				    }
				    if (!exists $rem_dupl_hash{$hkey} )
				    {
                                        $rem_dupl_hash{$hkey}  = $dataLine[$peg];
					undef $hkey;
				    }
			      }
			 } # grep 
			 else
			 {
#			     print "minMaxAvgArr = @minMaxAvg \n";
#			      print "into else part of loop for $temp_pegname \n";
                            $peg_value = $dataLine[$peg];
			    $to_match= $ucdataEntity . '-' . $varNames[$pegIndex];
			    $print_flag = (split(/\|/,$configHash{uc($to_match)}))[2];
                            $key_value="$ucdataEntity|$returnedKey|$outTstamp|$varNames[($pegIndex)]"."&&"."$ref_tstamp$result_hash{$g_template_name.':'.$temp_pegname}";


                            if ( defined $main_hash1{$key_value}{'max'} && $main_hash1{$key_value}{'max'} < $peg_value) 
			    {
                                $main_hash1{$key_value}{'max'} = $peg_value;
                                push @{$main_hash1{$key_value}{'values'}},$peg_value;
			       if ( "$debug" == 1 )
			       {
			          $debug_hkey=$g_template_name.'_'.$key_value;
			          $debug_hash{$debug_hkey}{'max'} = $peg_value;
				       undef $debug_hkey;
		                }
                            }
			    elsif ( defined $main_hash1{$key_value}{'min'} && $main_hash1{$key_value}{'min'} > $peg_value ) 
			    {
                                $main_hash1{$key_value}{'min'} = $peg_value;
                                push @{$main_hash1{$key_value}{'values'}},$peg_value;
			       if ( "$debug" == 1 )
			       {
			          $debug_hkey=$g_template_name.'_'.$key_value;
			          $debug_hash{$debug_hkey}{'min'} = $peg_value;
				       undef $debug_hkey;
		                }

                            }
                            $main_hash1{$key_value}{'max'} = $peg_value if ( not defined $main_hash1{ $key_value}{'max'} );
                            $main_hash1{$key_value}{'min'} = $peg_value if ( not defined $main_hash1{ $key_value}{'min'} );
			    if ( "$debug" == 1 )
			    {
			       $debug_hkey=$g_template_name.'_'.$key_value;
	                       $debug_hash{$debug_hkey}{'max'} = $peg_value if ( not defined $main_hash1{ $key_value}{'max'} );;
	                       $debug_hash{$debug_hkey}{'min'} = $peg_value if ( not defined $main_hash1{ $key_value}{'min'} );;
			       undef $debug_hkey;

			    }
                            push @{$main_hash1{$key_value}{'values'}},$peg_value if ( not defined $main_hash1{$key_value}{'values'} );
                            $main_hash1{$key_value}{'print_flag'} = [ split(//,$print_flag) ] if ( not defined $main_hash1{$key_value}{'print_flag'} );


                        } #else grep
		       } #if
			  $peg++;
		     } #while

		 }
                 
        }
($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
#print "preParser:End of main loop that does Pivot : $theTime \n"; 
next;
  }
  PPP:
}

foreach $key1 (keys % rem_dupl_hash)
{
 $rem_dupl_hash{$key1}=~s/\s*$//g;
 print OUTFILE $key1.'|'.$rem_dupl_hash{$key1};
 print OUTFILE "\n";
 }

if  ($debug == "1" )
{
foreach $dkey1 (keys % debug_hash)
{
 if (defined $debug_hash{$dkey1}{'max'} )
 {
   print DEBUGFILE $dkey1.'|'.$debug_hash{$dkey1}{'max'};
   print DEBUGFILE "\n";
 }
 if (defined $debug_hash{$dkey1}{'min'} )
 {
   print DEBUGFILE $dkey1.'|'.$debug_hash{$dkey1}{'min'};
 print DEBUGFILE "\n";
 }
 print DEBUGFILE $dkey1.'|'.$debug_hash{$dkey1};
 print DEBUGFILE "\n";
}
close DEBUGFILE;
}
($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
print "preParser:End of main loop and start procedure Find_min_max_avg start: $theTime \n"; 

find_print_min_max_avg ( \%main_hash1);

($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
print "preParser:Find_min_max_avg end: $theTime \n"; 

# This function is used to find the minimum,maximum and average and print it to outfile
sub find_print_min_max_avg($) {
	
	my $pmain_hash = shift;
	my $main_hash1 = %$pmain_hash;
	$count = 1;

($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
print "preParser:Find_min_max_avg First Loop Starts : $theTime \n"; 

	foreach $key ( sort keys %main_hash1 ) {
	   foreach ( @{$main_hash1{$key}{'print_flag'}} ) {
			@key_form = split('\|',$key);
			$ref_part_and_pegval= pop @key_form;
			@ref_part_arr=split('\&\&',$ref_part_and_pegval);

                        $ref_part=pop @ref_part_arr;
			$peg_name = pop @ref_part_arr;
			print OUTFILE $ref_part.'&&'.join('|',@key_form,'Min'.$peg_name,$main_hash1{$key}{'min'}) ,"\n" if  $_ eq 'M';
			print OUTFILE $ref_part.'&&'.join('|',@key_form,'Max'.$peg_name,$main_hash1{$key}{'max'}) ,"\n" if  $_ eq 'X';
			if ( $_ eq 'A' ) {
				$total_pegs = @{$main_hash1{$key}{'values'}};
				$sum = $sum + $_ foreach ( @{$main_hash1{$key}{'values'}} );
				print OUTFILE $ref_part.'&&'.join('|',@key_form,'Avg'.$peg_name,int($sum/$total_pegs)) , "\n";

				$sum = 0;
			}
		}
          }
($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";
print "preParser:Find_min_max_avg After Second Loop: $theTime \n"; 
}

	

###########################################
# Collect the NodeName - RNC or DOM
sub collectNodeName {
  my $nodeLine = $_[0];
  chomp $nodeLine;
  my @NodeNameLine = split(/=/,$nodeLine);
  $label = $NodeNameLine[1];
  return $label;
}

###########################################
# Build entity specific keys
sub buildKey {
   local ($entity) = $_[0];
   local ($instance) = $_[1];

# For all entities the DO_RNC_Id comes from $PROSPECT_HOME/msc_list
# rather than from the data
   #DO_RNC
   # jaya: We are trying to capture atDescriptor values in the RNC key so that 
   # we can distinguish atDesc=1 and 2 counters and load them into 2 different 
   # tables in ParserState.
   if ($entity =~ /^DO_RNC$/) { 

     #SEAde64393 Check for RNC_ResourceControl_R2.2/4.0 to make sure we add a special number to
     # help with loading this template in a different table /package in the ParserState 
     #Removing the fix for RNCResourceControl (:999) as we are no longer using the old parser
     # I am not sure what will happen to 2.2 templates but no customer is using EVDO2.2(All on 4.0)

     if ($instance =~ m/^(\d\d\/\d\d\/\d\d\d\d)/ && $g_templ_name=~m/RNC[_]?ResourceControl/) 
     {
       $keyInfo = "$DO_RNC_Id|$nodeName";

     }
     elsif ($instance =~ m/^(\d)$/) {
        $keyInfo = "$DO_RNC_Id|$nodeName:$instance";
     }
     else {
        $keyInfo = "$DO_RNC_Id|$nodeName";
     }
    }
###########################################
   #DO_RNC_Card
   #jaya:Change done for RNCPerfExtnBySlot where instance format is atDescriptor.slotNumber
   # or slotNumber.some_index
   # We want to pass just the slotNumber for the RNCSlotHash lookup.
   elsif ($entity =~ /^DO_RNC_Card$/) {
    my $slotInfo; # Declare the variable outside the if .. else
    my @slotArray;
     if ( $instance =~ m/(\d+).(\d+)$/) 
     {
       $val1 = $1;	
       $val2 = $2;	
#       if ($g_templ_name=~m/QoSPerfBySlot_R(\d)\.(\d)/) 
       if ($g_templ_name=~m/QoSPerfBySlot/) 
     {
       $slotVal = $val1;
       $rncSlotPktAppn=$val2;
       $slotInfo = $RNCSlotHash{$slotVal}; #static hash lookup info
       @slotArray="";
     @slotArray = split (/\|/, $slotInfo);
     $keyInfo = "$DO_RNC_Id|$slotArray[0]:R$rncSlotPktAppn";

     }
#     elsif ( $g_templ_name=~m/RNCPerfExtnBySlot_R(\d)\.(\d)/) 
     elsif ( $g_templ_name=~m/RNCPerfExtnBySlot/) 
     {
       $atDesc = $val1;
       $slotVal = $val2;
       $slotInfo = $RNCSlotHash{$slotVal}; #static hash lookup info
       @slotArray="";
     @slotArray = split (/\|/, $slotInfo);
     $keyInfo = "$DO_RNC_Id|$slotArray[0]:$atDesc";
     }
       else
       {
#      print "Going into RNSMLoadBalancing and Slot for atDescriptor\n";
       $slotVal= $val1;
       $slotIndex= $val2;
       $slotInfo = $RNCSlotHash{$slotVal}; #static hash lookup info
       @slotArray="";
     @slotArray = split (/\|/, $slotInfo);
     $keyInfo = "$DO_RNC_Id|$slotArray[0]";
       }
     }
     else
     {
        $slotInfo = $RNCSlotHash{$instance}; #static hash lookup info
        @slotArray = split (/\|/, $slotInfo);
        $keyInfo = "$DO_RNC_Id|$slotArray[0]";
     }
   }
###########################################
# New for EVDO R3.0
###########################################
#DO_RNC_Source
      elsif ($entity =~ /^DO_RNC_Source$/) {
	   $keyInfo = "$DO_RNC_Id|$instance";
      }
#########################################
   #DO_RNC_CPU
   elsif ($entity =~ /^DO_RNC_CPU$/) {
     my $slotInfo = $RNCCPUHash{$instance}; #static hash lookup info
     my @slotArray = split (/\|/, $slotInfo);
     $keyInfo = "$DO_RNC_Id|$slotArray[0]|$slotArray[1]";
   }
###########################################
   #DO_RNC_If
   elsif ($entity =~ /^DO_RNC_If$/) {
     # we assume that the DO_RNC_Label corresponds to the interface.name field 
#     print "DO_RNC_If instance = $instance and nodename=$nodeName \n";
     my $ifInfo = $IFhash{"$instance^^$nodeName"};#IF tab mapping info
#     print "ifInfo is $ifInfo \n";
     my @ifArray = split (/\|/, $ifInfo);
     my @ifNameArray = split (/\//, $ifArray[4]);
#     print "ifNameArray is @ifNameArray\n";
     #split out the slot name so the card id can be looked up in the RNCSlothash hash 
     my @longSlotArray = split (/_/,$ifNameArray[1]);
     my $slotNumOnly = $longSlotArray[1];
     my $slotInfo = $RNCSlotHash{$slotNumOnly};#static lookup info
     my @slotArray = split (/\|/, $slotInfo);
     $keyInfo = "$DO_RNC_Id|$slotArray[0]|$ifNameArray[3]|$ifNameArray[4]";
   }
###########################################
   #DOM
   # In this case the $nodeName is the dom name, not the rnc label
   elsif ($entity =~ /^DOM$/) {
     my $domInfo = $DOMhash{"$nodeName"};
     $domInfo =~ s/ //g; # ditch the spaces
     my @domArray = split (/\|/, $domInfo);
     $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; # get only the number out of the bts name
     # Fix for SEAde66135 (Files fail when we have a growth cells, domArray3 or bts id becomes nonnumeric and file
     # fails to load. We are going to filter such cells out
     if ( $domArray[3] =~ m/(\D)+/ ) { $keyInfo="";  return $keyInfo ; }
     $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$nodeAliasName"; 
   }
###########################################
   #DOM_CPU
   #assuming cpu id is instance 
   #Assumes that nodeName is the DOM in this case,for the lookup from %DOMhash
   elsif ($entity =~ /^DOM_CPU$/) {
     my $domInfo = $DOMhash{"$nodeName"};#DOM tab mapping info
     $domInfo =~ s/ //g; # ditch the spaces
     my @domArray = split (/\|/, $domInfo);
#     print "domInfo for DOM_CPU is $domInfo \n";
     $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; #bts number
     if ( $domArray[3] =~ m/(\D)+/ ) { $keyInfo="";  return $keyInfo ; }
     my $domCPUInfo = $DOMCPUHash{$instance}; #get static lookup info
     my @domCPUArray = split (/\|/, $domCPUInfo);
     $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$domCPUInfo";
   }
###########################################
   #DOM_If
   #Assumes that nodeName is the DOM in this case for the lookup from %DOMhash
   elsif ($entity =~ /^DOM_If$/) {
     my $domInfo = $DOMhash{"$nodeName"};#DOM tab mapping info
     $domInfo =~ s/ //g; # ditch the spaces
     my @domArray = split (/\|/, $domInfo);
     $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; #bts number
     if ( $domArray[3] =~ m/(\D)+/ ) { $keyInfo="";  return $keyInfo ; }
     my $ifInfo = $IFhash{"$instance^^$nodeAliasName"}; #IF tab mapping info
     $ifInfo =~ s/ //g; # ditch the spaces
     my @ifArray = split (/\|/, $ifInfo);
     my @ifNameArray = split (/\//, $ifArray[4]);
     #IPCS fix
     if ( $ifNameArray[5] eq '') 
     {
        $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$ifNameArray[2]|$ifNameArray[3]|$ifNameArray[4]";
     }
     else
     {
        $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$ifNameArray[2]|$ifNameArray[3]|$ifNameArray[4]_$ifNameArray[5]";
     }
   }
###########################################
   #DOM_Sector
   # should be uniquely identified by dom ip and pnoffset
   #Assume that dom ids are always 'DOM_<ip address>'
   elsif ($entity =~ /^DOM_Sector$/) {
     # get the pnoffset and ip address from the instance
     # if it's a file from the DOM and not the RNC, then it will 
     # only have the sector number for an instance
     # DOM ip will come from nodenam
      if (length($instance) < 12) {# data comes from DOM, short instance
	   my $sector = substr($instance,0,1);
       my $domip = $nodeName;
       $domip =~ s/DOM_//g; 
       my $domsectInfo = $DOMSECThash{"$sector^^$domip"};
       $domsectInfo =~ s/ //g; #ditch the spaces
       my @domsectArray = split(/\|/, $domsectInfo);
       my $domInfo = $DOMhash{"DOM_$domip"};# DOM mapping info 
       $domInfo =~ s/ //g; # ditch the spaces
       my @domArray = split (/\|/, $domInfo);
       $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; #bts number
       if ( $domArray[3] =~ m/(\D)+/ ) { $keyInfo="";  return $keyInfo ; }
	   my $element = $domsectArray[4];
       my $sector = $DOMSectHash{"$element"}; # sector letter from static hash

       if (length($sector) == 0) {
	 $sector = 0;
       }
       # changed for SEAde53513
 	$temp = $instance;
	#jaya : multiple instance change
        $temp =~ s/\./_/g;
        $keyInfo = "$DO_RNC_Id|$domArray[3]|$domsectArray[3]_$temp|$domsectArray[2]|$sector";
	 }else{ # data comes from RNC, has long instance. 
       my @instanceArray = split (/\./, $instance);
       my $domip = "$instanceArray[0]\.$instanceArray[1]\.$instanceArray[2]\.$instanceArray[3]";
       my $pnoffset = $instanceArray[5];
       my $domInfo = $DOMhash{"DOM_$domip"};# DOM mapping info 
       $domInfo =~ s/ //g; # ditch the spaces
       my @domArray = split (/\|/, $domInfo);
       $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; #bts number
       if ( $domArray[3] =~ m/(\D)+/ ) { $keyInfo="";  return $keyInfo ; }
	   my $is856Info = $IS856hash{"$pnoffset^^$domip"}; #IS856 mapping info
       my @is856Array = split (/\|/, $is856Info);
       $is856Array[4] =~ s/ //g; #ditch the spaces
       my $element = $is856Array[4]; 
       my $sector = $DOMSectHash{"$element"}; # sector letter from static hash
       if (length($sector) == 0) {
	 $sector = 0;
       }
     $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$instanceArray[4]|$sector";
     }
   }
###########################################
   #AirAbisPeer
   #DO_RNC_Label is DOM in this case
   elsif ($entity =~ /^AirAbisPeer$/) {
     my $domInfo = $DOMhash{"$nodeName"};#DOM tab mapping info
     $domInfo =~ s/ //g; # ditch the spaces
     my @domArray = split (/\|/, $domInfo);
     $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; #bts number
     if ( $domArray[3] =~ m/(\D)+/ ) { $keyInfo="";  return $keyInfo ; }
     $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$instance";
   }
###########################################
   #PDSN 
   elsif ($entity =~ /^DO_PDSN$/) {
     my $pdsnInfo = $PDSNhash{"${nodeName}^^$instance"};#PDSN tab mapping info
     my @pdsnArray = split (/\|/, $pdsnInfo);
     if (length($pdsnArray[1]) == 0) {
     $keyInfo = "";
     } else {
     $keyInfo = "$DO_RNC_Id|$pdsnArray[1]";
     }
   }
###########################################
   #DO_AAAServer
	  elsif ($entity =~ /^DO_AAAServer$/) {
     $keyInfo = "$DO_RNC_Id|$instance";
	  }
###########################################
    #DO_RNC_TrafficType
	  elsif ($entity =~ /^DO_RNC_TrafficType$/) {
     $keyInfo = "$DO_RNC_Id|$instance";
	  }
###########################################
   #DOM_RNC
     elsif ($entity =~ /^DOM_RNC$/) {
     my $domInfo = $DOMhash{"$nodeName"};#DOM tab mapping info
     $domInfo =~ s/ //g; # ditch the spaces
     my @domArray = split (/\|/, $domInfo);
     $domArray[3] =~ s/(.*?)([0-9]+)$/$2/; #bts number
     my @domCPUArray = split (/\|/, $domCPUInfo);
     $keyInfo = "$DO_RNC_Id|$domArray[3]|$domArray[4]|$instance";
   }
###########################################
   else {
     $keyInfo = "UNDEFINED KEY";
     print " I don't know this entity $entity ! \n";
     }
# final key scrubbing
$keyInfo =~ s/ //g;
$keyInfo = convertHyphen($keyInfo);
#print "Final keyinfo is $keyInfo \n";
return $keyInfo;
 }
###########################################
# build up the array of variable names
sub pushVarNameArray ($){
  $plocalLine=shift;
  $localLine=$$plocalLine;
#($second, $minute, $hour) = localtime();
#$theTime = "$hour:$minute:$second ";
#print "preParser:pushVarNames start :$theTime \n";
  @varname = split(/[()]/,$localLine);
  #Muthu
  if ( grep (/$varname[1]/i, keys %reference_peghash ) ) {
	  $temp_pegname = lc($varname[1]);
	  $result_hash{$g_template_name.':'.$temp_pegname} = $reference_peghash{lc($templateEntityHash{$g_template_name}).'-'.$temp_pegname}.$reference_templatehash{$g_template_name};
  }

  push (@varNames, ucfirst($varname[1]));

#($second, $minute, $hour) = localtime();
#$theTime = "$hour:$minute:$second ";
#print "preParser:pushVarNames end :$theTime \n";
#  print "Into pushVarNameArray VARNAMES = @varNames \n";
}

sub fillInBitMapArr($)
{
 $ucdataentity=shift;

    my @minMaxAvgArr = ();
    $index=0;
    foreach $varname (@varNames)
    {
       $to_match=$ucdataentity.'-'.$varname ;
       if ( grep ( /$to_match/i, keys %configHash1 ) )
       {
         $minMaxAvgArr[$index]=1;
#	 print "Position $index is set in fillInBitMap for $varname \n";
       }
       $index++;	 
    }
    $minMaxAvgHash{$g_template_name}{'Arr'} = \@minMaxAvgArr;

}
###########################################
# ensure the number of variables matches the number of pegs 
sub checkNumPegs {
  $dataLineElements = @dataLine;
#  print "dataLineElements = $dataLineElements \n";
#  print "dataline is and numPegs=$numPegs \n ";

  	    if ($dataLineElements ne $numPegs) 
	      { 
#If you need to troubleshoot manually edited data use this print below
	       print "Going into bad loop for this dataline \n";
	       @dataLine1=join("|",@dataLine);
	       print @dataLine1;
	die "Number of pegs in header($numPegs) doesn't match number of pegs in data ($dataLineElements \n";
	      }
}
############################################
# build a hash from one of four config tables
sub buildLookupHash {

my $cfgent = $_[0];
my %newH1;
my %newH;

# build some cool sql using RANK to get recent config info
# First column in each case will be the key for the resulting hash
 #and IFALIAS='DesMoines1_1/MC1900BTS13/8'
  if ($cfgent eq "DOM") {
    $sqltext = 
      "\(SELECT name || '||', id, tstamp, btsidstring, nodealiasname
 , RANK \(\) OVER \(PARTITION BY id ORDER BY tstamp DESC\) tsrank
 FROM nor_cfg_rnctab_dom_cmp WHERE tstamp <= '$fnTstamp'
 AND system_id = '$systemId'\)"
    }
  elsif ($cfgent eq "IF") {
    $sqltext = # need ifindex and ifalias to get distinct row 
      "\(SELECT ifindex ||'^^'|| ifalias || '||' cpdId, primaryip, ifalias, ifname, name
 , RANK \(\) OVER \(PARTITION BY id ORDER BY tstamp DESC\) tsrank
 FROM nor_cfg_rnctab_inter_cmp WHERE tstamp <= '$fnTstamp'
 AND system_id = '$systemId' 
 order by ifalias\)";
    }
  elsif ($cfgent eq "DO_RNC_If") {
    $sqltext = # need ifindex and ifalias to get distinct row 
      "\(SELECT ifindex ||'^^'|| 'RNC_'||primaryip || '||' cpdId, primaryip, ifalias, ifname, name
 , RANK \(\) OVER \(PARTITION BY id ORDER BY tstamp DESC\) tsrank
 FROM nor_cfg_rnctab_inter_cmp WHERE tstamp <= '$fnTstamp'
 AND system_id = '$systemId' 
 and 'RNC_'||primaryip= '$nodeName' order by cpdId\)";

    }
  elsif ($cfgent eq "IS856") {
    $sqltext = # need pnoffset and ip to get distinct row
      "\(SELECT pnoffset || '^^' || ipaddress || '||', ipaddress, is856carchanno, 
nodealiasname, sectelementname, carriername
 , RANK \(\) OVER \(PARTITION BY id ORDER BY tstamp DESC\) tsrank
 FROM nor_cfg_rnctab_is856_cmp WHERE tstamp <= '$fnTstamp'
 AND system_id = '$systemId'\)"
    }
  elsif ($cfgent eq "DO_PDSN") {
    $sqltext =
      "\(SELECT name || '^^'|| pdsnnumber || '||', pdsnipaddress, name
 , RANK \(\) OVER \(PARTITION BY substr(id, 0,instr(id,'-',1,3)-1) ORDER BY tstamp DESC\) tsrank
 FROM nor_cfg_rnctab_pdsn_cmp WHERE tstamp <= '$fnTstamp'
 AND system_id = '$systemId'\)"
    }
# need a new one for looking up dom sector info by
# DOM_IP and sectorElementIndex....
  elsif ($cfgent eq "DOMSECT") {
    $sqltext =
      "\(SELECT substr\(SECTELEMENTNAME,-1,1\) || '^^' || ipaddress || '||', 
ipaddress, is856carchanno, nodealiasname, sectelementname, carriername
, RANK \(\) OVER \(PARTITION BY id ORDER BY tstamp DESC\) tsrank
FROM nor_cfg_rnctab_is856_cmp WHERE tstamp <= '$fnTstamp'
 AND system_id = '$systemId'\)"
    }
#########
  else {
    die "Unsupported config lookup \n";
    }

#print "sqltext is \n";
#print $sqltext;
# Run the query
$sqlResult = `sqlplus -s $connectString \<<EOF
SET FEEDBACK OFF
SET TAB OFF
SET COLSEP |
SET PAGESIZE 0
SET LINESIZE 32767
ALTER SESSION SET NLS_DATE_FORMAT='YYYYMMDDHH24MI';
SELECT * FROM
$sqltext
WHERE tsrank = 1;
exit;
EOF`;
if ($cfgent eq "DOMSECT") {
}
  
if ($cfgent eq "IF") {
#print "==========\n";
#print "sqlResult \n";
#print "$sqlResult \n";
#print "==========\n";
@sqlResultArray = split (/\n/,$sqlResult);
%sqlResultHash = ( );
$ctr=0;
foreach $sqlResultArray (@sqlResultArray) {
#print "Result Arr in loop \n";
#print "$sqlResultArray[$ctr] \n";
  @splitsqlResultArray = split(/\|\|/,$sqlResultArray[$ctr]);
#  print "DO_RNC_IF resultArr $splitsqlResultArray[0] \n";
  chomp $splitsqlResultArray[1];
  @splitFirstPart=split(/\^\^/,$splitsqlResultArray[0]);
  @splitSecPart=split(/\|/,$splitsqlResultArray[1]);
  my @sArr=();
    $sArr[0]=$splitFirstPart[0]."^^RNC_".$splitSecPart[1]; 
    $sArr[1]=$splitsqlResultArray[1];
    $sArr[0]=~s/(\s)+//g; # Remove extra blanks 
    $RNCIP="RNC_".$splitSecPart[1];
    $RNCIP=~s/(\s)+//g;
#    print "RNCIP=${RNCIP}-- \n";
#    print "sArr 0=${sArr[0]}-- \n";
#    print "sArr 1=${sArr[1]}-- \n";
  if (!exists $newH1{$RNCIP})
  {
    $num=0;
    $newH1{$RNCIP}->{arr}->[$num]->{sarr}=\@sArr;
    $newH1{$RNCIP}->{cnt}=0;
#    print "FIRST CNT ->$newH1{$RNCIP}->{cnt} \n";
  }
  else
  {
    $newH1{$RNCIP}->{cnt}++;
#    print "NEXT CNT ->$newH1{$RNCIP}->{cnt} \n";
    my $num=$newH1{$RNCIP}->{cnt};
    $newH1{$RNCIP}->{arr}->[$num]->{sarr}=\@sArr;

   }

  $ctr++;
  next;
} # for sqlResultArray

%sqlResultHash = ( );
$ctr=0;
foreach $sqlResultArray (@sqlResultArray) {
#print "Result Arr in loop \n";
#print "$sqlResultArray[$ctr] \n";
  @splitsqlResultArray = split(/\|\|/,$sqlResultArray[$ctr]);
#  print "resultArr $splitsqlResultArray[0] \n";
  ($index,$ifalias)=split(/\^\^/,$splitsqlResultArray[0]);
#  print "index = $index and ifalias = $ifalias \n";
   my @sArr;
    $sArr[0]=$splitsqlResultArray[0];
    $sArr[1]=$splitsqlResultArray[1];

#    print "sArr 0= $sArr[0] \n";
#    print "sArr 1= $sArr[1] \n";
  if (!exists $newH{$ifalias})
  {
    $num=0;
    $newH{$ifalias}->{arr}->[$num]->{sarr}=\@sArr;
    $newH{$ifalias}->{cnt}=0;
#    print "FIRST CNT ->$newH{$ifalias}->{cnt} \n";
  }
  else
  {
    $newH{$ifalias}->{cnt}++;
#    print "NEXT CNT ->$newH{$ifalias}->{cnt} \n";
    my $num=$newH{$ifalias}->{cnt};
    $newH{$ifalias}->{arr}->[$num]->{sarr}=\@sArr;

   }
#    print "\nFIRST->$newH{$ifalias}->{arr}->[$num]->{sarr}->$sArr[0] \n";
#    print "FIRST->$newH{$ifalias}->{arr}->[$num]->{sarr}->$sArr[1] \n";
#    $newH{$ifalias}->{cnt}++; 
   $ctr++;
  next;
}
$retArr[0]=\%newH;
$retArr[1]=\%newH1;

return @retArr;
} # for cfgEnt=IF

# build a hash from the result
@sqlResultArray = split (/\n/,$sqlResult);
%sqlResultHash = ( );
my $ctr = 0; 
foreach $sqlResultArray (@sqlResultArray) {
  @splitsqlResultArray = split(/\|\|/,$sqlResultArray[$ctr]);
  $sqlResultHash{"$splitsqlResultArray[0]"} = $splitsqlResultArray[1];
  $ctr ++;
  next;
}
return %sqlResultHash;
}

############################################
# no hyphens in keys
sub convertHyphen
{
	my $in = $_[0] ;
	$in =~ s/\-/_/g;
	return $in ;      
}



